@@ -96,6 +96,7 @@ static void	audio_play_init		(AudioPlay *play);
 
 static gboolean	audio_play_playing_to_ready (AudioPlay *play);
 //static void	gst_play_dispose	(GObject *object);
+static void	callback_sink_eos 	(GstElement *element, AudioPlay *play);
 
 /* GObject type stuff */
 static GObject *parent_class = NULL;
@@ -149,6 +150,83 @@ audio_play_error (AudioPlay *play, const gchar *message)
 	if (play->priv->error_handler)
 		play->priv->error_handler (message, play->priv->error_data);
 }
+/* handler functions dispatched from iterator */
+
+static void
+handle_have_type (AudioPlay *play)
+{
+	AudioPlayPriv *priv = play->priv;
+	GstCaps *caps = priv->caps;
+	gchar *mime = g_strdup_printf (gst_caps_get_mime (caps));
+	GstElement *decoder = NULL;
+
+#ifdef DEBUG
+	fprintf (stderr, "handle_have_type\n");
+	fprintf (stderr, "have caps, mime type is %s\n", mime);
+#endif
+
+	g_assert (GST_IS_ELEMENT (priv->pipeline));
+	priv->have_type = FALSE;
+
+	gst_element_set_state (priv->pipeline, GST_STATE_READY);
+
+	/* disconnect the typefind from the pipeline and remove it,
+	 * since we now know what the type is */
+	g_print ("DEBUG: disconnecting typefind\n");
+	gst_element_disconnect (priv->src, priv->typefind);
+	/* ref it so it doesn't go away */
+	g_object_ref (priv->typefind);
+	g_print ("DEBUG: removing typefind\n");
+	gst_bin_remove (GST_BIN (priv->pipeline), priv->typefind);
+
+	/* gst_scheduler_show (GST_ELEMENT_SCHED (priv->pipeline)); */
+
+	/* now based on the mime type set up the pipeline properly */
+	g_print ("DEBUG: deciding on decoder\n");
+	if (strstr (mime, "mp3"))
+		decoder = gst_element_factory_make ("mad", "decoder");
+	else if (strstr (mime, "x-ogg"))
+		decoder = gst_element_factory_make ("vorbisfile", "decoder");
+	else if (strstr (mime, "x-wav"))
+		decoder = gst_element_factory_make ("wavparse", "decoder");
+	else if (strstr (mime, "x-flac"))
+		decoder = gst_element_factory_make ("flacdec", "decoder");
+	else if (strstr (mime, "x-mod") || strstr (mime, "x-s3m") || 
+	         strstr (mime, "x-xm") || strstr (mime, "x-it"))
+		decoder = gst_element_factory_make ("modplug", "decoder");
+	else
+	{
+		g_print ("mime type %s not handled in this program.\n", mime);
+		g_free (mime);
+		return;
+	}
+	g_free (mime);
+	priv->decoder = decoder;
+
+	/* set it to null */
+	/* somewhere between 0.4.2 and 0.5.0, setting this to NULL
+	 * broke threads completely because of the cothread freeing
+	 * or something to that effect
+	 * FIXME: figure out why that happened so we can set it to NULL
+	 * again */
+	gst_element_set_state (priv->pipeline, GST_STATE_NULL);
+
+	/* now put playback pipe in place of where typefind was  */
+	g_print ("DEBUG: putting in decoder\n");
+	gst_bin_add (GST_BIN (priv->pipeline), priv->decoder);
+	g_print ("DEBUG: putting in sink\n");
+	gst_bin_add (GST_BIN (priv->pipeline), priv->sink);
+	/* FIXME: this shouldn't go here */
+	g_signal_connect (G_OBJECT (play->priv->sink), "eos",
+			  G_CALLBACK (callback_sink_eos), play);
+	if (!gst_element_connect (priv->src, priv->decoder))
+		g_error ("warning: could not connect src and decoder !\n");
+	if (!gst_element_connect (priv->decoder, priv->sink))
+		g_print ("warning: could not connect decoder and sink !\n");
+	if (gst_element_set_state (priv->pipeline, GST_STATE_PAUSED)
+			== GST_STATE_FAILURE)
+		g_print ("pipelien could not be paused !\n");
+}
 
 /* callbacks & idlers */
 /* iterator idler func; iterates pipeline */
@@ -176,6 +254,8 @@ idler (AudioPlay *play)
 #ifdef DEBUG
 	g_print ("-");
 #endif
+	if (play->priv->have_type)
+		handle_have_type (play);
 	if (play->priv->have_eos)
 	{
 		g_print ("DEBUG: signalling eos\n");
@@ -220,7 +300,7 @@ tick_timeout (AudioPlay *play)
 }
 
 
-void
+static void
 callback_sink_eos (GstElement *element, AudioPlay *play)
 {
 	AudioPlayPriv *priv = play->priv;
@@ -279,74 +359,23 @@ deep_notify (GObject *object, GstObject *origin,
 	g_print ("DEBUG: deep_notify: %s\n", pspec->name);
 }
 /* callback for when we have the type of the file
+ * set the boolean and handle it in the iterator
  * we set up a decoder based on the mime type
  */
 static void
 have_type (GstElement *element, GstCaps *caps, AudioPlay *play)
 {
 	AudioPlayPriv *priv = play->priv;
-	gchar *mime = g_strdup_printf (gst_caps_get_mime (caps));
-	GstElement *decoder = NULL;
 
 #ifdef DEBUG
 	g_print ("DEBUG: have_type callback called.\n");
-	fprintf (stderr, "have caps, mime type is %s\n", mime);
 #endif
 
 	g_assert (GST_IS_ELEMENT (priv->pipeline));
-	gst_element_set_state (priv->pipeline, GST_STATE_READY);
-
-	/* disconnect the typefind from the pipeline and remove it,
-	 * since we now know what the type is */
-	gst_element_disconnect (priv->src, priv->typefind);
-	/* ref it so it doesn't go away */
-	g_object_ref (priv->typefind);
-	gst_bin_remove (GST_BIN (priv->pipeline), priv->typefind);
-
-	/* gst_scheduler_show (GST_ELEMENT_SCHED (priv->pipeline)); */
-
-	/* now based on the mime type set up the pipeline properly */
-	if (strstr (mime, "mp3"))
-		decoder = gst_element_factory_make ("mad", "decoder");
-	else if (strstr (mime, "x-ogg"))
-		decoder = gst_element_factory_make ("vorbisfile", "decoder");
-	else if (strstr (mime, "x-wav"))
-		decoder = gst_element_factory_make ("wavparse", "decoder");
-	else if (strstr (mime, "x-flac"))
-		decoder = gst_element_factory_make ("flacdec", "decoder");
-	else
-	{
-		g_print ("mime type %s not handled in this program.\n", mime);
-		g_free (mime);
-		return;
-	}
-	g_free (mime);
-	priv->decoder = decoder;
-
-	/* set it to null */
-	/* somewhere between 0.4.2 and 0.5.0, setting this to NULL
-	 * broke threads completely because of the cothread freeing
-	 * or something to that effect
-	 * FIXME: figure out why that happened so we can set it to NULL
-	 * again */
-	gst_element_set_state (priv->pipeline, GST_STATE_NULL);
-
-	/* now put playback pipe in place of where typefind was  */
-	g_print ("DEBUG: putting in decoder\n");
-	gst_bin_add (GST_BIN (priv->pipeline), priv->decoder);
-	g_print ("DEBUG: putting in sink\n");
-	gst_bin_add (GST_BIN (priv->pipeline), priv->sink);
-	/* FIXME: this shouldn't go here */
-	g_signal_connect (G_OBJECT (play->priv->sink), "eos",
-			  G_CALLBACK (callback_sink_eos), play);
-	if (!gst_element_connect (priv->src, priv->decoder))
-		g_error ("warning: could not connect src and decoder !\n");
-	if (!gst_element_connect (priv->decoder, priv->sink))
-		g_print ("warning: could not connect decoder and sink !\n");
-	if (gst_element_set_state (priv->pipeline, GST_STATE_PAUSED)
-			== GST_STATE_FAILURE)
-		g_print ("pipelien could not be paused !\n");
+	priv->have_type = TRUE;
+	priv->caps = caps;
 }
+
 /* object functions */
 
 static void
@@ -539,7 +568,7 @@ audio_play_autoplug (AudioPlay *play, GError **error)
 	audio_play_debug (play);
 	g_print ("DEBUG: starting iterating for autoplug\n");
 	g_assert (gst_element_get_state (play->priv->pipeline) == GST_STATE_PLAYING);
-	while (play->priv->decoder == NULL)
+	while (play->priv->have_type == FALSE)
 	{
 		g_print ("*");
 	       if (!gst_bin_iterate (GST_BIN (play->priv->pipeline)))
@@ -557,7 +586,8 @@ audio_play_autoplug (AudioPlay *play, GError **error)
 		       return FALSE;
 	       }
 	}
-	/* we have the type */
+	/* we have the type, handle it */
+	handle_have_type (play);
 #ifdef DEBUG
 	g_print ("DEBUG: done autoplug\n");
 #endif
@@ -619,7 +649,7 @@ audio_play_update_length (AudioPlay *play)
 	if (caps == NULL) g_print ("WARNING: caps are NULL\n");
 	gst_caps_debug (caps, "caps of decoder src pad");
 
-	res = gst_pad_query (pad, GST_PAD_QUERY_TOTAL, &format, &value);
+	res = gst_pad_query (pad, GST_QUERY_TOTAL, &format, &value);
 	if (!res) 
 	{
 		g_print ("WARNING: pad query failed,can't get length !\n");
@@ -661,6 +691,10 @@ audio_play_seek_to_time (AudioPlay *play, guint64 nanosecs)
 			       audio_play_signals [TICK_SIGNAL], 0,
 			       play->priv->nanoseconds);
 	}
+	else
+	{
+		g_error ("DEBUG: gst_element_send_event: FAILED\n");
+	}
 	gst_element_set_state (play->priv->pipeline, GST_STATE_PLAYING);
 }
 
